/*
 
  copyright (c) 2004, Michal Salaban <emes@pld-linux.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2 as
  published by the Free Software Foundation (see file COPYING for details).

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
*/

/* $Id: la.c,v 1.7 2004/02/05 18:56:24 emes Exp $ */

#include "parseopt/parseopt.h"
#include "exit.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>

#define         BUFSIZE     64

#define OPT_LA1     1
#define OPT_LA5     2
#define OPT_LA15    3
#define OPT_STEP    4
#define OPT_RUN     5
#define OPT_ONCE    6
#define OPT_CONJ    7

#define OPT_NONE    0

const int   true = 1, false = 0;

optrepr     repr[] = {
    {   OPT_LA1,    O_DOUBLE,   "la1"   },
    {   OPT_LA5,    O_DOUBLE,   "la5"   },
    {   OPT_LA15,   O_DOUBLE,   "la15"  },
    {   OPT_STEP,   O_INT,      "step"  },
    {   OPT_RUN,    O_STRING,   "run"   },
    {   OPT_ONCE,   O_BOOL,     "once"  },
    {   OPT_CONJ,   O_BOOL,     "all"   },
    {   OPT_NONE,   0,          NULL    }
};

optval      sta[] = {
    {   OPT_LA1,    NULL    },
    {   OPT_LA5,    NULL    },
    {   OPT_LA15,   NULL    },
    {   OPT_STEP,   NULL    },
    {   OPT_RUN,    NULL    },
    {   OPT_ONCE,   NULL    },
    {   OPT_CONJ,   NULL    },
    {   OPT_NONE,   NULL    }
};

const char      *lafile = "/proc/loadavg";
const char      *usage  =
                "Usage: sysfence-la OPTION=VALUE ... \n"
                "\n"
                "Where OPTION can be:\n"
                "\n"
                "   la1         load average during last minute\n"
                "   la5         load average during last 5 minutes\n"
                "   la15        load average during last 15 minutes\n"
                "   step        how often probe for LA (seconds, default = 10)\n"
                "   run         command to run when LA exceeds thresholds\n"
                "   once        on  = invoke command only once after threshold has been exceeded\n"
                "               off = run command every step seconds if threshold has been\n"
                "                     exceeded\n"
                "               (use: on/off, 1/0, yes/no values) default is on\n"
                "    all        on  = invoke command if ALL limits have been exceeded\n"
                "               off = invoke command if ANY limit has been exceeded.\n"
                "               (use: on/off, 1/0, yes/no values) default is off\n"
                "\n"
                "The 'run' option is MANDATORY.\n"
                "At least one of 'laXX' options must be specified.\n"
                "\n"
                "Report bugs to <emes@pld-linux.org>\n"
                "\n";

double          limit[3],
                load[3],
                chk[3];

char            *command = NULL;
char            fbuf[BUFSIZE];
int             fha;
long int        step = 10;
int             once = 1, conjunction = 0;

void main_loop ()
{
    int     tmp;
    int     i;
    int     done = 0, hit = 0;
    char    *one, *two, *three;

    while (true) {
        lseek (fha, 0, SEEK_SET);
        read (fha, (char *) &fbuf, BUFSIZE);

        one   = &fbuf[0];
        two   = index (one, ' ') + 1;
        three = index (two, ' ') + 1;
        
        load[0] = atof (one);
        load[1] = atof (two);
        load[2] = atof (three);

        if (! conjunction) {
            /* alternative: set hit to 1 if ANY limit has been exceeded */
            hit = 0;
            for (i = 0; i < 3; i++)
                if (chk[i] && (load[i] > limit[i])) hit = 1;
        } else {
            /* conjunction: set hit to 0 if ANY limit has NOT beed exceeded */
            hit = 1;
            for (i = 0; i < 3; i++)
                if (chk[i] && (load[i] <= limit[i])) hit = 0;
        }
            
        if ((hit == 1) && ((done == 0) || (once == 0))) system (command);

#ifdef DEBUG
        printf ("%ld: hit=%d done=%d, step=%d once=%d, "
                "la1=%0.2f la5=%0.2f la15=%0.2f, "
                "th1=%0.2f th5=%0.2f th15=%0.2f",
                time (0),
                hit, done,
                step, once,
                load[0], load[1], load[2],
                limit[0], limit[1], limit[2]
                );
        if ((hit != 0) && ((done == 0) || (once == 0))) printf (", executed: %s", command);
        printf ("\n");
#endif
        
        done = hit;
        
        sleep (step);
    }
}

int main (int argc, char *argv[])
{
    int     i, limset;
    int     cbs = 0, ccs = 0, slen;
    void    *opt[3], *three, *stepopt, *onceopt, *conjopt;
    
    if (! fetch_opt_array (&sta, &repr, OPT_NONE, argc - 1, &argv[1]))
        bail_out (EXIT_NOCONF, NULL);

    opt[0]  = get_val (&sta, OPT_NONE, OPT_LA1);
    opt[1]  = get_val (&sta, OPT_NONE, OPT_LA5);
    opt[2]  = get_val (&sta, OPT_NONE, OPT_LA15);
    stepopt = get_val (&sta, OPT_NONE, OPT_STEP);
    command = get_val (&sta, OPT_NONE, OPT_RUN);
    onceopt = get_val (&sta, OPT_NONE, OPT_ONCE);
    conjopt = get_val (&sta, OPT_NONE, OPT_CONJ);

    limset = false;
    for (i = 0; i < 3; i++) {
        if (opt[i] != NULL) {
            limit[i] = *((double *) opt[i]);
            chk[i] = true;
            limset = true;
        } else {
            limit[i] = 0.0;
            chk[i] = false;
        }
    }
        
    if (stepopt != NULL) step = *((long int *) stepopt);
    if (onceopt != NULL) once = *((int *) onceopt);
    if (conjopt != NULL) conjunction = *((int *) conjopt);
    
    /* command and at least one of thresholds must be given */
    if ((!limset) || (command == NULL)) 
        bail_out (EXIT_NOCONF, NULL);

    /* open lafile (usually /proc/loadavg) */
    fha = open (lafile, O_RDONLY);
    if (fha < 0) bail_out (EXIT_IO, lafile);
    
#ifndef DEBUG
    if (daemon (0, 0) == 0) main_loop ();
    else bail_out (EXIT_IO, "daemon()");
#endif
#ifdef DEBUG
    main_loop ();
#endif

    return 0;
}

/* $Id: la.c,v 1.7 2004/02/05 18:56:24 emes Exp $ */
